///////////////////////////////////////////
//this function changes the moving side in the current position
///////////////////////////////////////////
void ChangeMovingSide(void);


///////////////////////////////////////////
//this function prepares everything for the start
///////////////////////////////////////////
bool Initialize(void);


///////////////////////////////////////////
//this function returns the number of a piece on the square
//or -1 if the square is empty.
///////////////////////////////////////////
int WhichPiece(int square);


///////////////////////////////////////////
//this function returns the number of square, on which the king of 
//the desired side stays.
//Assumes that the king is present!!!
///////////////////////////////////////////
int WhereKing(int side);


///////////////////////////////////////////
//this function shows the current chessboard in "numberous" style, 
//i.e. white king = 1, white queen = 2, empty square = -1
///////////////////////////////////////////
void ShowBoard(void);


///////////////////////////////////////////
//this function returns the number of attacks 
//on the desired square by the desired side
///////////////////////////////////////////
int IsAttacked(int square, int ByWhichSide);


///////////////////////////////////////////
//this function adds the desired piece onto the desired square
///////////////////////////////////////////
void AddPiece(int square, int piece);


///////////////////////////////////////////
//this function removes certain piece from the mentioned square
///////////////////////////////////////////
void RemovePiece(int square, int piece);


///////////////////////////////////////////
//this function creates the initial chess board position
///////////////////////////////////////////
void InitialPosition();


///////////////////////////////////////////
//this function determines the type of the move
///////////////////////////////////////////
int SetMoveType(int fromSquare, int toSquare);


///////////////////////////////////////////
//This function makes the move,
//Type must match to the real type of fromSquare-toSquare move.
//So the advice is to always use Type = SetMoveType(fromSquare,toSquare)
///////////////////////////////////////////
bool Move(int fromSquare, int toSquare, int Type);


///////////////////////////////////////////
//this function calculates the first hash key of the current position
///////////////////////////////////////////
__int32 ComputePrimaryHashKey(void);


///////////////////////////////////////////
//this function calculates the second hash key of the current position
///////////////////////////////////////////
__int32 ComputeSecondaryHashKey(void);


///////////////////////////////////////////
//this function computes all legal moves of a given piece 
//on a given square in the current position
///////////////////////////////////////////
__int64 ComputeMoves(int piece, int square);


///////////////////////////////////////////
//this function loads evaluation parameters from a file.
//order is important (see thesis)!
///////////////////////////////////////////
bool LoadParameters( int *Chr);


///////////////////////////////////////////
//this function checks if current position is valid
///////////////////////////////////////////
bool PositionValidity(void);


///////////////////////////////////////////
//this function starts playing with a console opponent
//WhoStarts = true --> compuetr starts, WhoStarts = false --> opponent starts
//Depth = search depth 
///////////////////////////////////////////
void PlayAgainstOpponent(bool, int);


///////////////////////////////////////////
//this function makes the program play against itself.
//behaviour is exactly the same as in PlayAgainstOpponent
///////////////////////////////////////////
void AutoPlay(int Depth);


///////////////////////////////////////////
//SOME FUNCTIONS NOT USED AT THE FINAL VERSION
//BUT WRITTEN FOR SOME PURPOSES WHILE CREATING
///////////////////////////////////////////


///////////////////////////////////////////
//this function prints the bit board in the following manner:
//
//a1...h1
//.......
//a8...h8
//That's why Inverse...
///////////////////////////////////////////
void InverseBitRepresentation( __int64 x);


///////////////////////////////////////////
//this function transforms square number to its usual notation,
//i.e. 0-->a1, 63-->h8
///////////////////////////////////////////
bool NumberToField(int number, char *field);


///////////////////////////////////////////
//this function transforms usual notation to square number,
//i.e. a1-->0, h8-->63
///////////////////////////////////////////
int FieldToNumber(char *field);


///////////////////////////////////////////
//this function makes the program select any valid move for each side
//for "plies" times or until mate/stalemate happens
///////////////////////////////////////////
void RandomPlaying(int plies);