#include "stdafx.h"

bool WhiteLongCastling,
	 WhiteShortCastling,
	 BlackLongCastling,
	 BlackShortCastling;
/*
Castling possibilities
true  = not done, but still possible
false = either done already or not possible any more
*/


bool WhiteCastlingDone,
	 BlackCastlingDone;
/*
Indicates if castling was preformed or not.
true and false, respectively
*/


int MovingSide;
/*
Indcates, which side is on move
0=White
7=Black
*/


__int64 Squares[64];
/*
The chess board squares (bit-board style)

Squares[0]= "A1"
Squares[1]= "B1"
...
Squares[62]= "G8"
Squares[63]= "H8"
*/


int MoveType;
/*
The type of the move to be made

Ordinary		   Move = 0
Capture			   Move = 1
Castling		   Move = 2
En Passant Capture Move = 3
Promotion Move:
Ordianry    toQueen  = 401
		    toRook   = 402
		    toBishop = 403
		    toKnight = 404
WithCapture toQueen  = 411
			toRook   = 412
			toBishop = 413
			toKnight = 414
*/


int EnPassant;
/*
Indicates the vertical, on which en passant capture move
is possible right now
*/


int _50MovesDraw = 0;
//Controls the rule of "50 moves draw"


TranspositionTable TransTable[TT_Size];
/*
Declares transposition table

*********REMINDER*********
struct TranspositionTable 
	{
		int PHK,
			SHK;
		
		int Depth, 
			Value;
	};

Depth is a number of plies, on which the position's score was calculated
If there is no entry in the table, the corresponding Depth is set to -1.
*/


aMove MoveList[MoveHistorySize];
/*
Declares array of moves

*********REMINDER*********
struct aMove
{
	int fromSquare,
		toSquare,

		Type,
		AttackStatus,
		Score;
};
**************************
*/


aMove GoodMoves[5],
	  BadMoves[5];
//Corresponding best and worst moves of the previous search


int PositionCounter = 0;
//Just for statistics.
//Counts the number of evaluated positions


__int64 BitBoards[14];
/*
BitBoards:

BitBoard[0] = "ALL WHITE PIECES"
BitBoard[1] = "WHITE KING"
BitBoard[2] = "ALL WHITE QUEENS"
BitBoard[3] = "ALL WHITE ROOKS"
BitBoard[4] = "ALL WHITE BISHOPS"
BitBoard[5] = "ALL WHITE KNIGHTS"
BitBoard[6] = "ALL WHITE PAWNS"

BitBoard[7] = "ALL BLACK PIECES"
BitBoard[8] = "BLACK KING"
BitBoard[9] = "ALL BLACK QUEENS"
BitBoard[10]= "ALL BLACK ROOKS"
BitBoard[11]= "ALL BLACK BISHOPS"
BitBoard[12]= "ALL BLACK KNIGHTS"
BitBoard[13]= "ALL BLACK PAWNS"

Index of this array also stands for the referring 
to a corresponding piece in certain cases.
Empty square is assigned -1 then.
*/


int Chromosome[26];
/*
Evaluation parameters (in a particular order: see thesis).
Loaded from the file "Chr.txt" every time the program starts.
*/


//BitBoards of Moves (see below)
//Introduced to simplify calculations in some cases
__int64       KingMovesBB[64]={
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0,
								0,0,0,0,0,0,0,0},
			  KnightMovesBB[64]={
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0},
			   RookMovesBB[64]={
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0},
			   BishopMovesBB[64]={
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0,
								  0,0,0,0,0,0,0,0};


///////////////////////////////////////////
//Next there are the arrays of all possible moves of every piece from each square.
//No special one for Queen, since it is Rook and Bishop combined.
//No one for pawn either: pawn moves are simple enough to determine them directly.
///////////////////////////////////////////
int KingMoves[64][8]={
	
	{ 1, 8, 9,-1,-1,-1,-1,-1},//A1
	{ 0, 8, 9,10, 2,-1,-1,-1},
	{ 1, 9,10,11, 3,-1,-1,-1},
	{ 2,10,11,12, 4,-1,-1,-1},
	{ 3,11,12,13, 5,-1,-1,-1},
	{ 4,12,13,14, 6,-1,-1,-1},
	{ 5,13,14,15, 7,-1,-1,-1},
	{ 6,14,15,-1,-1,-1,-1,-1},//H1

	{16,17, 9, 1, 0,-1,-1,-1},//A2
	{ 8,16,17,18,10, 2, 1, 0},
	{ 9,17,18,19,11, 3, 2, 1},
	{10,18,19,20,12, 4, 3, 2},
	{11,19,20,21,13, 5, 4, 3},
	{12,20,21,22,14, 6, 5, 4},
	{13,21,22,23,15, 7, 6, 5},
	{14,22,23, 7, 6,-1,-1,-1},//H2

	{24,25,17, 9, 8,-1,-1,-1},//A3
	{16,24,25,26,18,10, 9, 8},
	{17,25,26,27,19,11,10, 9},
	{18,26,27,28,20,12,11,10},
	{19,27,28,29,21,13,12,11},
	{20,28,29,30,22,14,13,12},
	{21,29,30,31,23,15,14,13},
	{22,30,31,15,14,-1,-1,-1},//H3

	{32,33,25,17,16,-1,-1,-1},//A4
	{24,32,33,34,26,18,17,16},
	{25,33,34,35,27,19,18,17},
	{26,34,35,36,28,20,19,18},
	{27,35,36,37,29,21,20,19},
	{28,36,37,38,30,22,21,20},
	{29,37,38,39,31,23,22,21},
	{30,38,39,23,22,-1,-1,-1},//H4

	{40,41,33,25,24,-1,-1,-1},//A5
	{32,40,41,42,34,26,25,24},
	{33,41,42,43,35,27,26,25},
	{34,42,43,44,36,28,27,26},
	{35,43,44,45,37,29,28,27},
	{36,44,45,46,38,30,29,28},
	{37,45,46,47,39,31,30,29},
	{38,46,47,31,30,-1,-1,-1},//H5

	{48,49,41,33,32,-1,-1,-1},//A6
	{40,48,49,50,42,34,33,32},
	{41,49,50,51,43,35,34,33},
	{42,50,51,52,44,36,35,34},
	{43,51,52,53,45,37,36,35},
	{44,52,53,54,46,38,37,36},
	{45,53,54,55,47,39,38,37},
	{46,54,55,39,38,-1,-1,-1},//H6

	{56,57,49,41,40,-1,-1,-1},//A7
	{48,56,57,58,50,42,41,40},
	{49,57,58,59,51,43,42,41},
	{50,58,59,60,52,44,43,42},
	{51,59,60,61,53,45,44,43},
	{52,60,61,62,54,46,45,44},
	{53,61,62,63,55,47,46,45},
	{54,62,63,47,46,-1,-1,-1},//H7

	{57,49,48,-1,-1,-1,-1,-1},//A8
	{56,58,50,49,48,-1,-1,-1},
	{57,59,51,50,49,-1,-1,-1},
	{58,60,52,51,50,-1,-1,-1},
	{59,61,53,52,51,-1,-1,-1},
	{60,62,54,53,52,-1,-1,-1},
	{61,63,55,54,53,-1,-1,-1},
	{62,55,54,-1,-1,-1,-1,-1} //H8
};

int KnightMoves[64][8]={

	{17,10,-1,-1,-1,-1,-1,-1},//A1
	{16,18,11,-1,-1,-1,-1,-1},
	{ 8,17,19,12,-1,-1,-1,-1},
	{ 9,18,20,13,-1,-1,-1,-1},
	{10,19,21,14,-1,-1,-1,-1},
	{11,20,22,15,-1,-1,-1,-1},
	{12,21,23,-1,-1,-1,-1,-1},
	{13,22,-1,-1,-1,-1,-1,-1},//H1

	{25,18, 2,-1,-1,-1,-1,-1},//A2
	{24,26,19, 3,-1,-1,-1,-1},
	{16,25,27,20, 4, 0,-1,-1},
	{17,26,28,21, 5, 1,-1,-1},
	{18,27,29,22, 6, 2,-1,-1},
	{19,28,30,23, 7, 3,-1,-1},
	{20,29,31, 4,-1,-1,-1,-1},
	{21,30, 5,-1,-1,-1,-1,-1},//H2

	{33,26,10, 1,-1,-1,-1,-1},//A3
	{32,34,27,11, 2, 0,-1,-1},
	{24,33,35,28,12, 3, 1, 8},
	{25,34,36,29,13, 4, 2, 9},
	{26,35,37,30,14, 5, 3,10},
	{27,36,38,31,15, 6, 4,11},
	{28,37,39, 7, 5,12,-1,-1},
	{29,38, 6,13,-1,-1,-1,-1},//H3

	{41,34,18, 9,-1,-1,-1,-1},//A4
	{40,42,35,19,10, 8,-1,-1},
	{32,41,43,36,20,11, 9,16},
	{33,42,44,37,21,12,10,17},
	{34,43,45,38,22,13,11,18},
	{35,44,46,39,23,14,12,19},
	{36,45,47,15,13,20,-1,-1},
	{37,46,14,21,-1,-1,-1,-1},//H4

	{49,42,26,17,-1,-1,-1,-1},//A5
	{48,50,43,27,18,16,-1,-1},
	{40,49,51,44,28,19,17,24},
	{41,50,52,45,29,20,18,25},
	{42,51,53,46,30,21,19,26},
	{43,52,54,47,31,22,20,27},
	{44,53,55,23,21,28,-1,-1},
	{45,54,22,29,-1,-1,-1,-1},//H5

	{57,50,34,25,-1,-1,-1,-1},//A6
	{56,58,51,35,26,24,-1,-1},
	{48,57,59,52,36,27,25,32},
	{49,58,60,53,37,28,26,33},
	{50,59,61,54,38,29,27,34},
	{51,60,62,55,39,30,28,35},
	{52,61,63,31,29,36,-1,-1},
	{53,62,30,37,-1,-1,-1,-1},//H6

	{58,42,33,-1,-1,-1,-1,-1},//A7
	{59,43,34,32,-1,-1,-1,-1},
	{56,60,44,35,33,40,-1,-1},
	{57,61,45,36,34,41,-1,-1},
	{58,62,46,37,35,42,-1,-1},
	{59,63,47,38,36,43,-1,-1},
	{60,39,37,44,-1,-1,-1,-1},
	{61,38,45,-1,-1,-1,-1,-1},//H7

	{50,41,-1,-1,-1,-1,-1,-1},//A8
	{51,42,40,-1,-1,-1,-1,-1},
	{52,43,41,48,-1,-1,-1,-1},
	{53,44,42,49,-1,-1,-1,-1},
	{54,45,43,50,-1,-1,-1,-1},
	{55,46,44,51,-1,-1,-1,-1},
	{47,45,52,-1,-1,-1,-1,-1},
	{46,53,-1,-1,-1,-1,-1,-1} //H8

};
int RookMoves[64][4][7]={

/*
	The directions:

	{	 {LEFT},
		{RIGHT},
		   {UP},
		{DOWN}},//FIELD

	*/
	{	{-1,-1,-1,-1,-1,-1,-1},
		{ 1, 2, 3, 4, 5, 6, 7},
		{ 8,16,24,32,40,48,56},
		{-1,-1,-1,-1,-1,-1,-1}},//A1

	{	{ 0,-1,-1,-1,-1,-1,-1},
		{ 2, 3, 4, 5, 6, 7,-1},
		{ 9,17,25,33,41,49,57},
		{-1,-1,-1,-1,-1,-1,-1}},//B1

	{	{ 1, 0,-1,-1,-1,-1,-1},
		{ 3, 4, 5, 6, 7,-1,-1},
		{10,18,26,34,42,50,58},
		{-1,-1,-1,-1,-1,-1,-1}},//C1

	{	{ 2, 1, 0,-1,-1,-1,-1},
		{ 4, 5, 6, 7,-1,-1,-1},
		{11,19,27,35,43,51,59},
		{-1,-1,-1,-1,-1,-1,-1}},//D1

	{	{ 3, 2, 1, 0,-1,-1,-1},
		{ 5, 6, 7,-1,-1,-1,-1},
		{12,20,28,36,44,52,60},
		{-1,-1,-1,-1,-1,-1,-1}},//E1

	{	{ 4, 3, 2, 1, 0,-1,-1},
		{ 6, 7,-1,-1,-1,-1,-1},
		{13,21,29,37,45,53,61},
		{-1,-1,-1,-1,-1,-1,-1}},//F1

	{	{ 5, 4, 3, 2, 1, 0,-1},
		{ 7,-1,-1,-1,-1,-1,-1},
		{14,22,30,38,46,54,62},
		{-1,-1,-1,-1,-1,-1,-1}},//G1

	{	{ 6, 5, 4, 3, 2, 1, 0},
		{-1,-1,-1,-1,-1,-1,-1},
		{15,23,31,39,47,55,63},
		{-1,-1,-1,-1,-1,-1,-1}},//H1

	{	{-1,-1,-1,-1,-1,-1,-1},
		{ 9,10,11,12,13,14,15},
		{ 16,24,32,40,48,56,-1},
		{ 0,-1,-1,-1,-1,-1,-1}},//A2

	{	{ 8,-1,-1,-1,-1,-1,-1},
		{10,11,12,13,14,15,-1},
		{17,25,33,41,49,57,-1},
		{ 1,-1,-1,-1,-1,-1,-1}},//B2

	{	{ 9, 8,-1,-1,-1,-1,-1},
		{11,12,13,14,15,-1,-1},
		{18,26,34,42,50,58,-1},
		{ 2,-1,-1,-1,-1,-1,-1}},//C2

	{	{10, 9, 8,-1,-1,-1,-1},
		{12,13,14,15,-1,-1,-1},
		{19,27,35,43,51,59,-1},
		{ 3,-1,-1,-1,-1,-1,-1}},//D2

	{	{11,10, 9, 8,-1,-1,-1},
		{13,14,15,-1,-1,-1,-1},
		{20,28,36,44,52,60,-1},
		{ 4,-1,-1,-1,-1,-1,-1}},//E2

	{	{12,11,10, 9, 8,-1,-1},
		{14,15,-1,-1,-1,-1,-1},
		{21,29,37,45,53,61,-1},
		{ 5,-1,-1,-1,-1,-1,-1}},//F2

	{	{13,12,11,10, 9, 8,-1},
		{15,-1,-1,-1,-1,-1,-1},
		{22,30,38,46,54,62,-1},
		{ 6,-1,-1,-1,-1,-1,-1}},//G2

	{	{14,13,12,11,10, 9, 8},
		{-1,-1,-1,-1,-1,-1,-1},
		{23,31,39,47,55,63,-1},
		{ 7,-1,-1,-1,-1,-1,-1}},//H2

	{	{-1,-1,-1,-1,-1,-1,-1},
		{17,18,19,20,21,22,23},
		{24,32,40,48,56,-1,-1},
		{ 8, 0,-1,-1,-1,-1,-1}},//A3

	{	{16,-1,-1,-1,-1,-1,-1},
		{18,19,20,21,22,23,-1},
		{25,33,41,49,57,-1,-1},
		{ 9, 1,-1,-1,-1,-1,-1}},//B3

	{	{17,16,-1,-1,-1,-1,-1},
		{19,20,21,22,23,-1,-1},
		{26,34,42,50,58,-1,-1},
		{10, 2,-1,-1,-1,-1,-1}},//C3

	{	{18,17,16,-1,-1,-1,-1},
		{20,21,22,23,-1,-1,-1},
		{27,35,43,51,59,-1,-1},
		{11, 3,-1,-1,-1,-1,-1}},//D3

	{	{19,18,17,16,-1,-1,-1},
		{21,22,23,-1,-1,-1,-1},
		{28,36,44,52,60,-1,-1},
		{12, 4,-1,-1,-1,-1,-1}},//E3

	{	{20,19,18,17,16,-1,-1},
		{22,23,-1,-1,-1,-1,-1},
		{29,37,45,53,61,-1,-1},
		{13, 5,-1,-1,-1,-1,-1}},//F3

	{	{21,20,19,18,17,16,-1},
		{23,-1,-1,-1,-1,-1,-1},
		{30,38,46,54,62,-1,-1},
		{14, 6,-1,-1,-1,-1,-1}},//G3

	{	{22,21,20,19,18,17,16},
		{-1,-1,-1,-1,-1,-1,-1},
		{31,39,47,55,63,-1,-1},
		{15, 7,-1,-1,-1,-1,-1}},//H3

	{	{-1,-1,-1,-1,-1,-1,-1},
		{25,26,27,28,29,30,31},
		{32,40,48,56,-1,-1,-1},
		{16, 8, 0,-1,-1,-1,-1}},//A4

	{	{24,-1,-1,-1,-1,-1,-1},
		{26,27,28,29,30,31,-1},
		{33,41,49,57,-1,-1,-1},
		{17, 9, 1,-1,-1,-1,-1}},//B4

	{	{25,24,-1,-1,-1,-1,-1},
		{27,28,29,30,31,-1,-1},
		{34,42,50,58,-1,-1,-1},
		{18,10, 2,-1,-1,-1,-1}},//C4

	{	{26,25,24,-1,-1,-1,-1},
		{28,29,30,31,-1,-1,-1},
		{35,43,51,59,-1,-1,-1},
		{19,11, 3,-1,-1,-1,-1}},//D4

	{	{27,26,25,24,-1,-1,-1},
		{29,30,31,-1,-1,-1,-1},
		{36,44,52,60,-1,-1,-1},
		{20,12, 4,-1,-1,-1,-1}},//E4

	{	{28,27,26,25,24,-1,-1},
		{30,31,-1,-1,-1,-1,-1},
		{37,45,53,61,-1,-1,-1},
		{21,13, 5,-1,-1,-1,-1}},//F4

	{	{29,28,27,26,25,24,-1},
		{31,-1,-1,-1,-1,-1,-1},
		{38,46,54,62,-1,-1,-1},
		{22,14, 6,-1,-1,-1,-1}},//G4

	{	{30,29,28,27,26,25,24},
		{-1,-1,-1,-1,-1,-1,-1},
		{39,47,55,63,-1,-1,-1},
		{23,15, 7,-1,-1,-1,-1}},//H4

	{	{-1,-1,-1,-1,-1,-1,-1},
		{33,34,35,36,37,38,39},
		{40,48,56,-1,-1,-1,-1},
		{24,16, 8, 0,-1,-1,-1}},//A5

	{	{32,-1,-1,-1,-1,-1,-1},
		{34,35,36,37,38,39,-1},
		{41,49,57,-1,-1,-1,-1},
		{25,17, 9, 1,-1,-1,-1}},//B5

	{	{33,32,-1,-1,-1,-1,-1},
		{35,36,37,38,39,-1,-1},
		{42,50,58,-1,-1,-1,-1},
		{26,18,10, 2,-1,-1,-1}},//C5

	{	{34,33,32,-1,-1,-1,-1},
		{36,37,38,39,-1,-1,-1},
		{43,51,59,-1,-1,-1,-1},
		{27,19,11, 3,-1,-1,-1}},//D5

	{	{35,34,33,32,-1,-1,-1},
		{37,38,39,-1,-1,-1,-1},
		{44,52,60,-1,-1,-1,-1},
		{28,20,12, 4,-1,-1,-1}},//E5

	{	{36,35,34,33,32,-1,-1},
		{38,39,-1,-1,-1,-1,-1},
		{45,53,61,-1,-1,-1,-1},
		{29,21,13, 5,-1,-1,-1}},//F5

	{	{37,36,35,34,33,32,-1},
		{39,-1,-1,-1,-1,-1,-1},
		{46,54,62,-1,-1,-1,-1},
		{30,22,14, 6,-1,-1,-1}},//G5

	{	{38,37,36,35,34,33,32},
		{-1,-1,-1,-1,-1,-1,-1},
		{47,55,63,-1,-1,-1,-1},
		{31,23,15, 7,-1,-1,-1}},//H5

	{	{-1,-1,-1,-1,-1,-1,-1},
		{41,42,43,44,45,46,47},
		{48,56,-1,-1,-1,-1,-1},
		{32,24,16, 8, 0,-1,-1}},//A6

	{	{40,-1,-1,-1,-1,-1,-1},
		{42,43,44,45,46,47,-1},
		{49,57,-1,-1,-1,-1,-1},
		{33,25,17, 9, 1,-1,-1}},//B6

	{	{41,40,-1,-1,-1,-1,-1},
		{43,44,45,46,47,-1,-1},
		{50,58,-1,-1,-1,-1,-1},
		{34,26,18,10, 2,-1,-1}},//C6

	{	{42,41,40,-1,-1,-1,-1},
		{44,45,46,47,-1,-1,-1},
		{51,59,-1,-1,-1,-1,-1},
		{35,27,19,11, 3,-1,-1}},//D6

	{	{43,42,41,40,-1,-1,-1},
		{45,46,47,-1,-1,-1,-1},
		{52,60,-1,-1,-1,-1,-1},
		{36,28,20,12, 4,-1,-1}},//E6

	{	{44,43,42,41,40,-1,-1},
		{46,47,-1,-1,-1,-1,-1},
		{53,61,-1,-1,-1,-1,-1},
		{37,29,21,13, 5,-1,-1}},//F6

	{	{45,44,43,42,41,40,-1},
		{47,-1,-1,-1,-1,-1,-1},
		{54,62,-1,-1,-1,-1,-1},
		{38,30,22,14, 6,-1,-1}},//G6

	{	{46,45,44,43,42,41,40},
		{-1,-1,-1,-1,-1,-1,-1},
		{55,63,-1,-1,-1,-1,-1},
		{39,31,23,15, 7,-1,-1}},//H6

	{	{-1,-1,-1,-1,-1,-1,-1},
		{49,50,51,52,53,54,55},
		{56,-1,-1,-1,-1,-1,-1},
		{40,32,24,16, 8, 0,-1}},//A7

	{	{48,-1,-1,-1,-1,-1,-1},
		{50,51,52,53,54,55,-1},
		{57,-1,-1,-1,-1,-1,-1},
		{41,33,25,17, 9, 1,-1}},//B7

	{	{49,48,-1,-1,-1,-1,-1},
		{51,52,53,54,55,-1,-1},
		{58,-1,-1,-1,-1,-1,-1},
		{42,34,26,18,10, 2,-1}},//C7

	{	{50,49,48,-1,-1,-1,-1},
		{52,53,54,55,-1,-1,-1},
		{59,-1,-1,-1,-1,-1,-1},
		{43,35,27,19,11, 3,-1}},//D7

	{	{51,50,49,48,-1,-1,-1},
		{53,54,55,-1,-1,-1,-1},
		{60,-1,-1,-1,-1,-1,-1},
		{44,36,28,20,12, 4,-1}},//E7

	{	{52,51,50,49,48,-1,-1},
		{54,55,-1,-1,-1,-1,-1},
		{61,-1,-1,-1,-1,-1,-1},
		{45,37,29,21,13, 5,-1}},//F7

	{	{53,52,51,50,49,48,-1},
		{55,-1,-1,-1,-1,-1,-1},
		{62,-1,-1,-1,-1,-1,-1},
		{46,38,30,22,14, 6,-1}},//G7

	{	{54,53,52,51,50,49,48},
		{-1,-1,-1,-1,-1,-1,-1},
		{63,-1,-1,-1,-1,-1,-1},
		{47,39,31,23,15, 7,-1}},//H7

	{	{-1,-1,-1,-1,-1,-1,-1},
		{57,58,59,60,61,62,63},
		{-1,-1,-1,-1,-1,-1,-1},
		{48,40,32,24,16, 8, 0}},//A8

	{	{56,-1,-1,-1,-1,-1,-1},
		{58,59,60,61,62,63,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{49,41,33,25,17, 9, 1}},//B8

	{	{57,56,-1,-1,-1,-1,-1},
		{59,60,61,62,63,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{50,42,34,26,18,10, 2}},//C8

	{	{58,57,56,-1,-1,-1,-1},
		{60,61,62,63,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{51,43,35,27,19,11, 3}},//D8

	{	{59,58,57,56,-1,-1,-1},
		{61,62,63,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{52,44,36,28,20,12, 4}},//E8

	{	{60,59,58,57,56,-1,-1},
		{62,63,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{53,45,37,29,21,13, 5}},//F8

	{	{61,60,59,58,57,56,-1},
		{63,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{54,46,38,30,22,14, 6}},//G8

	{	{62,61,60,59,58,57,56},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{55,47,39,31,23,15, 7}} //H8

};

int BishopMoves[64][4][7]={

/*
	The directions:
	
	{	 {LEFT-UP},
		 {RIGHT-UP},
		 {RIGHT-DOWN},
		 {LEFT-DOWN}},//FIELD

	*/	
	{	{-1,-1,-1,-1,-1,-1,-1},
		{ 9,18,27,36,45,54,63},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A1

	{	{ 8,-1,-1,-1,-1,-1,-1},
		{10,19,28,37,46,55,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//B1

	{	{ 9,16,-1,-1,-1,-1,-1},
		{11,20,29,38,47,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//C1

	{	{10,17,24,-1,-1,-1,-1},
		{12,21,30,39,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//D1

	{	{11,18,25,32,-1,-1,-1},
		{13,22,31,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//E1

	{	{12,19,26,33,40,-1,-1},
		{14,23,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//F1

	{	{13,20,27,34,41,48,-1},
		{15,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//G1

	{	{14,21,28,35,42,49,56},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//H1

	{	{-1,-1,-1,-1,-1,-1,-1},
		{17,26,35,44,53,62,-1},
		{ 1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A2

	{	{16,-1,-1,-1,-1,-1,-1},
		{18,27,36,45,54,63,-1},
		{ 2,-1,-1,-1,-1,-1,-1},
		{ 0,-1,-1,-1,-1,-1,-1}},//B2

	{	{17,24,-1,-1,-1,-1,-1},
		{19,28,37,46,55,-1,-1},
		{ 3,-1,-1,-1,-1,-1,-1},
		{ 1,-1,-1,-1,-1,-1,-1}},//C2

	{	{18,25,32,-1,-1,-1,-1},
		{20,29,38,47,-1,-1,-1},
		{ 4,-1,-1,-1,-1,-1,-1},
		{ 2,-1,-1,-1,-1,-1,-1}},//D2

	{	{19,26,33,40,-1,-1,-1},
		{21,30,39,-1,-1,-1,-1},
		{ 5,-1,-1,-1,-1,-1,-1},
		{ 3,-1,-1,-1,-1,-1,-1}},//E2

	{	{20,27,34,41,48,-1,-1},
		{22,31,-1,-1,-1,-1,-1},
		{ 6,-1,-1,-1,-1,-1,-1},
		{ 4,-1,-1,-1,-1,-1,-1}},//F2

	{	{21,28,35,42,49,56,-1},
		{23,-1,-1,-1,-1,-1,-1},
		{ 7,-1,-1,-1,-1,-1,-1},
		{ 5,-1,-1,-1,-1,-1,-1}},//G2

	{	{22,29,36,43,50,57,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{ 6,-1,-1,-1,-1,-1,-1}},//H2

	{	{-1,-1,-1,-1,-1,-1,-1},
		{25,34,43,52,61,-1,-1},
		{ 9, 2,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A3

	{	{24,-1,-1,-1,-1,-1,-1},
		{26,35,44,53,62,-1,-1},
		{10, 3,-1,-1,-1,-1,-1},
		{ 8,-1,-1,-1,-1,-1,-1}},//B3

	{	{25,32,-1,-1,-1,-1,-1},
		{27,36,45,54,63,-1,-1},
		{11, 4,-1,-1,-1,-1,-1},
		{ 9, 0,-1,-1,-1,-1,-1}},//C3

	{	{26,33,40,-1,-1,-1,-1},
		{28,37,46,55,-1,-1,-1},
		{12, 5,-1,-1,-1,-1,-1},
		{10, 1,-1,-1,-1,-1,-1}},//D3

	{	{27,34,41,48,-1,-1,-1},
		{29,38,47,-1,-1,-1,-1},
		{13, 6,-1,-1,-1,-1,-1},
		{11, 2,-1,-1,-1,-1,-1}},//E3

	{	{28,35,42,49,56,-1,-1},
		{30,39,-1,-1,-1,-1,-1},
		{14, 7,-1,-1,-1,-1,-1},
		{12, 3,-1,-1,-1,-1,-1}},//F3

	{	{29,36,43,50,57,-1,-1},
		{31,-1,-1,-1,-1,-1,-1},
		{15,-1,-1,-1,-1,-1,-1},
		{13, 4,-1,-1,-1,-1,-1}},//G3

	{	{30,37,44,51,58,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{14, 5,-1,-1,-1,-1,-1}},//H3

	{	{-1,-1,-1,-1,-1,-1,-1},
		{33,42,51,60,-1,-1,-1},
		{17,10, 3,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A4

	{	{32,-1,-1,-1,-1,-1,-1},
		{34,43,52,61,-1,-1,-1},
		{18,11, 4,-1,-1,-1,-1},
		{16,-1,-1,-1,-1,-1,-1}},//B4

	{	{33,40,-1,-1,-1,-1,-1},
		{35,44,53,62,-1,-1,-1},
		{19,12, 5,-1,-1,-1,-1},
		{17, 8,-1,-1,-1,-1,-1}},//C4

	{	{34,41,48,-1,-1,-1,-1},
		{36,45,54,63,-1,-1,-1},
		{20,13, 6,-1,-1,-1,-1},
		{18, 9, 0,-1,-1,-1,-1}},//D4

	{	{35,42,49,56,-1,-1,-1},
		{37,46,55,-1,-1,-1,-1},
		{21,14, 7,-1,-1,-1,-1},
		{19,10, 1,-1,-1,-1,-1}},//E4

	{	{36,43,50,57,-1,-1,-1},
		{38,47,-1,-1,-1,-1,-1},
		{22,15,-1,-1,-1,-1,-1},
		{20,11, 2,-1,-1,-1,-1}},//F4

	{	{37,44,51,58,-1,-1,-1},
		{39,-1,-1,-1,-1,-1,-1},
		{23,-1,-1,-1,-1,-1,-1},
		{21,12, 3,-1,-1,-1,-1}},//G4

	{	{38,45,52,59,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{22,13, 4,-1,-1,-1,-1}},//H4

	{	{-1,-1,-1,-1,-1,-1,-1},
		{41,50,59,-1,-1,-1,-1},
		{25,18,11, 4,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A5

	{	{40,-1,-1,-1,-1,-1,-1},
		{42,51,60,-1,-1,-1,-1},
		{26,19,12, 5,-1,-1,-1},
		{24,-1,-1,-1,-1,-1,-1}},//B5

	{	{41,48,-1,-1,-1,-1,-1},
		{43,52,61,-1,-1,-1,-1},
		{27,20,13, 6,-1,-1,-1},
		{25,16,-1,-1,-1,-1,-1}},//C5

	{	{42,49,56,-1,-1,-1,-1},
		{44,53,62,-1,-1,-1,-1},
		{28,21,14, 7,-1,-1,-1},
		{26,17, 8,-1,-1,-1,-1}},//D5

	{	{43,50,57,-1,-1,-1,-1},
		{45,54,63,-1,-1,-1,-1},
		{29,22,15,-1,-1,-1,-1},
		{27,18, 9, 0,-1,-1,-1}},//E5

	{	{44,51,58,-1,-1,-1,-1},
		{46,55,-1,-1,-1,-1,-1},
		{30,23,-1,-1,-1,-1,-1},
		{28,19,10, 1,-1,-1,-1}},//F5

	{	{45,52,59,-1,-1,-1,-1},
		{47,-1,-1,-1,-1,-1,-1},
		{31,-1,-1,-1,-1,-1,-1},
		{29,20,11, 2,-1,-1,-1}},//G5

	{	{46,53,60,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{30,21,12, 3,-1,-1,-1}},//H5

	{	{-1,-1,-1,-1,-1,-1,-1},
		{49,58,-1,-1,-1,-1,-1},
		{33,26,19,12, 5,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A6

	{	{48,-1,-1,-1,-1,-1,-1},
		{50,59,-1,-1,-1,-1,-1},
		{34,27,20,13, 6,-1,-1},
		{32,-1,-1,-1,-1,-1,-1}},//B6

	{	{49,56,-1,-1,-1,-1,-1},
		{51,60,-1,-1,-1,-1,-1},
		{35,28,21,14, 7,-1,-1},
		{33,24,-1,-1,-1,-1,-1}},//C6

	{	{50,57,-1,-1,-1,-1,-1},
		{52,61,-1,-1,-1,-1,-1},
		{36,29,22,15,-1,-1,-1},
		{34,25,16,-1,-1,-1,-1}},//D6

	{	{51,58,-1,-1,-1,-1,-1},
		{53,62,-1,-1,-1,-1,-1},
		{37,30,23,-1,-1,-1,-1},
		{35,26,17, 8,-1,-1,-1}},//E6

	{	{52,59,-1,-1,-1,-1,-1},
		{54,63,-1,-1,-1,-1,-1},
		{38,31,-1,-1,-1,-1,-1},
		{36,27,18, 9, 0,-1,-1}},//F6

	{	{53,60,-1,-1,-1,-1,-1},
		{55,-1,-1,-1,-1,-1,-1},
		{39,-1,-1,-1,-1,-1,-1},
		{37,28,19,10, 1,-1,-1}},//G6

	{	{54,61,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{38,29,20,11, 2,-1,-1}},//H6

	{	{-1,-1,-1,-1,-1,-1,-1},
		{57,-1,-1,-1,-1,-1,-1},
		{41,34,27,20,13, 6,-1},
		{-1,-1,-1,-1,-1,-1,-1}},//A7

	{	{56,-1,-1,-1,-1,-1,-1},
		{58,-1,-1,-1,-1,-1,-1},
		{42,35,28,21,14, 7,-1},
		{40,-1,-1,-1,-1,-1,-1}},//B7

	{	{57,-1,-1,-1,-1,-1,-1},
		{59,-1,-1,-1,-1,-1,-1},
		{43,36,29,22,15,-1,-1},
		{41,32,-1,-1,-1,-1,-1}},//C7

	{	{58,-1,-1,-1,-1,-1,-1},
		{60,-1,-1,-1,-1,-1,-1},
		{44,37,30,23,-1,-1,-1},
		{42,33,24,-1,-1,-1,-1}},//D7

	{	{59,-1,-1,-1,-1,-1,-1},
		{61,-1,-1,-1,-1,-1,-1},
		{45,38,31,-1,-1,-1,-1},
		{43,34,25,16,-1,-1,-1}},//E7

	{	{60,-1,-1,-1,-1,-1,-1},
		{62,-1,-1,-1,-1,-1,-1},
		{46,39,-1,-1,-1,-1,-1},
		{44,35,26,17, 8,-1,-1}},//F7

	{	{61,-1,-1,-1,-1,-1,-1},
		{63,-1,-1,-1,-1,-1,-1},
		{47,-1,-1,-1,-1,-1,-1},
		{45,36,27,18, 9, 0,-1}},//G7

	{	{62,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{46,37,28,19,10, 1,-1}},//H7

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{49,42,35,28,21,14, 7},
		{-1,-1,-1,-1,-1,-1,-1}},//A8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{50,43,36,29,22,15,-1},
		{48,-1,-1,-1,-1,-1,-1}},//B8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{51,44,37,30,23,-1,-1},
		{49,40,-1,-1,-1,-1,-1}},//C8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{52,45,38,31,-1,-1,-1},
		{50,41,32,-1,-1,-1,-1}},//D8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{53,46,39,-1,-1,-1,-1},
		{51,42,33,24,-1,-1,-1}},//E8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{54,47,-1,-1,-1,-1,-1},
		{52,43,34,25,16,-1,-1}},//F8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{55,-1,-1,-1,-1,-1,-1},
		{53,44,35,26,17, 8,-1}},//G8

	{	{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{-1,-1,-1,-1,-1,-1,-1},
		{54,45,36,27,18, 9, 0}} //H8

};