///////////////////////////////////////////
//this structure is used to store current position parameters
///////////////////////////////////////////
struct Params 
	{
		int fromSquare,
			dep,
			toSquare,
			dest,
			MoveType,
			EnPassant,
			MovingSide;
		
		bool WhiteLongCastling,
			 WhiteShortCastling,
			 BlackLongCastling,
			 BlackShortCastling,
			 WhiteCastlingDone,
			 BlackCastlingDone;
	};


///////////////////////////////////////////
//variables for the best found move
///////////////////////////////////////////
extern int BestFromSquare,
		   BestToSquare;


///////////////////////////////////////////
//This function takes one move back.
//It must be given an array of positional statements (of type Params),
//which the chessboard had a move ago
///////////////////////////////////////////
void MoveBack(Params);


///////////////////////////////////////////
//This function performs the full search, without cutting any branch
///////////////////////////////////////////
int FullSearch(int PliesToSearch);


///////////////////////////////////////////
//This function performs the alpha(a)-beta(b) search to a fixed depth.
//No Iterative Deepening!
///////////////////////////////////////////
int AB_Search(int PliesToSearch, int a, int b);


///////////////////////////////////////////
//this function uses Shannon's type-A strategy, 
//i.e. all top level moves are considered
///////////////////////////////////////////
int FindMove(int PliesToSearch);


///////////////////////////////////////////
//this function checks, 
//if a position in question is in the transposition table
///////////////////////////////////////////
bool isInTransTable(int PrHashKey, int SecHashKey, int PliesToSearch);


///////////////////////////////////////////
//this function adds a position in question to the transposition table
///////////////////////////////////////////
void AddToTransTable(int PrimaryHashKey, int SecondaryHashKey, int Score, int Depth);


///////////////////////////////////////////
//this function generates the moves for the current position
///////////////////////////////////////////
void GenerateMoves(aMove*, int);


///////////////////////////////////////////
//this function shows moves generated
///////////////////////////////////////////
void ShowMoves(aMove*);


///////////////////////////////////////////
//this function shows outstanding moves of the previous search
///////////////////////////////////////////
void ShowHistory(void);
